/************************** SunDst ***************************
 * by Ewald Kendziorra sr, 2016 www.kendziorra.nl            * 
 * Before you start: read the SunDst information in "info.h" *
 *************************************************************/

#ifndef RISE_SET_DST
#define RISE_SET_DST

#include <TimeLib.h>  // https://github.com/PaulStoffregen/Time

#define NORTHERN    // TEST Europe Northern hemisphere - uncomment this for TEST Windhoek, Namibia - southern hemisphere
                    //             ========              =========                                   ========

/** lat (Latitude) and lon (Longitude)
 *  Decimal degrees and plus/minus-North/South-East/West Latitude and longitude.
 *  The latitude is preceded by a minus sign ( – ) if it is south of the equator (a positive number implies north),
 *  and the longitude is preceded by a minus sign if it is west of the prime meridian (a positive number implies east)
 *  Latitude is also used to test if it's DST/Summer Time or not (Northern or Southern hemisphere).
 **/

/** DST/Summer Time rules. Change it for a other area **
 ** ================================================= **/
#ifdef NORTHERN    // TEST Northern hemisphere: Europe
/** EU summer time rules:
 *    start: last Sunday in March   at 01:00 add      1h   to local time
 *    end:   last Sunday in October at 01:00 subtract 1h   from local time
 **/
const uint8_t startMonth    =  3, endMonth    = 10;  // 1 - 12 (jan - dec)
const uint8_t startMonthDay =  0, endMonthDay =  0;  // 1 - 31, not used = 0 (if using ...WeekDay)
const int8_t  startWeek     = -1, endWeek     = -1;  // 1 -  4, last week of month: -1 (4 or 5)
const uint8_t startWeekDay  =  1, endWeekDay  =  1;  // 1 -  7 (Sun - Sat), not used = 0 (if using ...MonthDay)
const uint8_t startHour     =  1, endHour     =  1;  // 0 - 23 (UTC)
const uint8_t startMinute   =  0, endMinute   =  0;  // 0 - 59 (Lord Howe Island rules use a half-hour time zone ;=))
//
const float lat = 52.0, lon = 5.0;    // TEST Northern hemisphere: centre of the Netherlands
const uint16_t dstOffset =   3600;  // dst in seconds
const double   timeZone =    1;  // UTC−12 to UTC+14 (Lord Howe Island rules use a half-hour time zone ;=))
//===========================
#else    // TEST Southern hemisphere: Windhoek, Namibia
/** Windhoek, Namibia DST rules:
 *    start: first Sunday in September at 00:00 add      1h   to local time
 *    end:   first Sunday in April     at 01:00 subtract 1h   from local time
 **/
const uint8_t startMonth    =  9, endMonth    =  4;
const uint8_t startMonthDay =  0, endMonthDay =  0;
const int8_t  startWeek     =  1, endWeek     =  1;
const uint8_t startWeekDay  =  1, endWeekDay  =  1;
const uint8_t startHour     =  1, endHour     =  0;  // (UTC)
const uint8_t startMinute   =  0, endMinute   =  0;
//
const float lat = -22.566667, lon = 17.083333;  // Southern hemisphere: Windhoek, Namibia
const uint16_t dstOffset =   3600;
const double   timeZone =    1;
//===========================
#endif

float zenith = 90.883;
// zenith
//    offical      = 90.883 degrees (90° 50')
//    civil        = 96 degrees
//    nautical     = 102 degrees
//    astronomical = 108 degrees

const float toRad = 0.01745329, toDeg = 57.2957786;
const uint16_t oneHour = 3600;
const uint32_t oneDay  = 86400, oneWeek = 604800;

#endif

