/************************** SunDst ***************************
 * by Ewald Kendziorra sr, 2016 www.kendziorra.nl            * 
 * Before you start: read the SunDst information in "info.h" *
 *************************************************************/

#include <Streaming.h>  // http://arduiniana.org/libraries/streaming
#include "SunDst.h"
#include "sun.h"
#include "dst.h"

// ########################### DST rules and location in SunDst.h ##########################
// change to European or Windhoek Namibia in SunDst.h or set your own DST rules and location   
// #########################################################################################

char str[40];  // sprintf() print buf

void setup() {
  Serial.begin(115200);
  
  setTime(12, 0, 0, 15, 1, 2016);    // set UTC time (use this time for the demo)
  time_t n = now();  // get a UNIX timestamp from now(), we use this timestamp for the whole demo
  // you can also get a timestamp with my dst.h function: 
  //                             hh mm ss  DD MM  YYYY
  //    time_t n = calcTimeStamp(12, 0, 0, 15, 1, 2016);
  
  /** intro **/
  Serial << F("Sunrise and Sunset on location.\nDaylight Saving Time (DST) (Worldwide).\n");
  Serial << F("  The rules are easy to modify.\n  Flex or fixed with 6 Start/End values.\n\n");
  
  /** location **/
  if (lat >= 0) Serial << F("TEST Northern hemisphere: Europe\n");
  else Serial << F("TEST Southern hemisphere: Windhoek, Namibia\n");
  Serial << F("Latitude : ") << ftostr(lat, 8, 4) << F("\nLongitude: ") << ftostr(lon, 8, 4) << endl;
  Serial << F("Timezone : ") << timeZone << F("\tdecimal hours - there are also\n\t\thalf Hour and 45-Minute Time Zones\nDSToffset: ") << dstOffset << "\tseconds\n\t\tthere are also half Hour DST offsets\n";
  
  /** time **/
  Serial << year(n) << F("       dd-mm-yyyy hh:mm\n");
  
  /** today **/
  sprintf(str, "now      : %02d-%02d-%02d %02d:%02d UTC\n", day(n), month(n), year(n), hour(n), minute(n));
  Serial << str;
  printNow(n);
  
  /** sunRise/Set time calculation from "time_t now()" **/
  calcSun();
  printSun();
  
  /** dst/summer time calculation from "time_t now()" **/
  calcDst(); 
  printDst();
  printIsDst(n);  // dst/summer time ?

  /**  sunRise/Set time calculation today + 26 weeks **/
  n += 26 * oneWeek;  // + 26 * one week in seconds (is 26 weeks after now() from start)
  isDST = true;  // simulate DST/Summertime
  calcSun(n);
  sprintf(str, "\n+26 weeks: %02d-%02d-%02d %02d:%02d UTC\n", day(n), month(n), year(n), hour(n), minute(n));
  Serial << str;
  printNow(n);
  printSun();
  printIsDst(n);  // dst/summer time ?
  
  /** dst/summer time calculation with "int year" **/
  n +=  26 * oneWeek;  // + 26 * one week in seconds (is 52 weeks after now() from start)
  calcDst(n);
  Serial << endl << year(n) << endl;
  sprintf(str, "+52 weeks: %02d-%02d-%02d %02d:%02d UTC\n", day(n), month(n), year(n), hour(n), minute(n));
  Serial << str;
  printNow(n);
  printDst();
  printIsDst(n);  // dst/summer time ?
}

/** UTC time to Local time **/
time_t localTime(time_t t) {
  calcDst(t);
  return t + (uint32_t)(timeZone * oneHour + (isDST ? dstOffset : 0)); 
}

void printNow(time_t t) {
  time_t l = localTime(t);
  sprintf(str, "now      :            %02d:%02d Local\n", hour(l), minute(l));
  Serial << str;
}

void printDst() {
  char _str[40];
  time_t localTime = timeZone * oneHour;
  dstStart += localTime;              // UNIX timestamp START
  dstEnd   += localTime + dstOffset;  // UNIX timestamp END

  sprintf(str, "DST start: %02d-%02d-%02d %02d:%02d Local\n", day(dstStart), month(dstStart), year(dstStart), hour(dstStart), minute(dstStart));
  sprintf(_str, "DST end  : %02d-%02d-%02d %02d:%02d Local\n", day(dstEnd), month(dstEnd), year(dstEnd), hour(dstEnd), minute(dstEnd));
  if (lat >= 0) Serial << str << _str;
  else Serial << _str << str;    
}

void printIsDst(time_t t) {
  Serial << "DST now? : "<< (isDST ? "Yes" : "No") << endl;      // dst/summer time?
}

void printSun() {
  sprintf(str, "sunRise  :            %02d:%02d Local\n", sunRiseHour, sunRiseMinute); 
  Serial << str;
  sprintf(str, "sunSet   :            %02d:%02d Local\n", sunSetHour,  sunSetMinute); 
  Serial << str;
}

void loop() {}
