/************************** SunDst ***************************
 * by Ewald Kendziorra sr, 2016 www.kendziorra.nl            * 
 * Before you start: read the SunDst information in "info.h" *
 *************************************************************/

#include "SunDst.h"
#include "sun.h"

/** today **/
void calcSun() {
  calcSun(now());
}

/** from date **/
void calcSun(time_t t) {
  /** calc yearDay **/
  uint16_t N = floor(275 * month(t) / 9) - (floor((month(t) + 9) / 12) * (1 + floor((year(t) - 4 * floor(year(t) / 4) + 2) / 3)))  + day(t) - 30;

  /** get SunRise time **/
  calcDst(t);  // set isDST
  float sunRiseDec = sunTime(N, true);
  sunRiseDec += timeZone + (isDST ? (dstOffset / 3600) : 0);  // to local time
  sunRiseHour = sunRiseDec;  // byte<-float ;=))
  sunRiseMinute = (sunRiseDec - sunRiseHour) * 60;

  /** get SunSet time **/
  float sunSetDec = sunTime(N, false);
  sunSetDec += timeZone + (isDST ? (dstOffset / 3600) : 0);  // to local time
  sunSetHour = sunSetDec;
  sunSetMinute = (sunSetDec - sunSetHour) * 60;
}

/** calc decimal time **/
float sunTime(uint16_t N, boolean rise) {
  float lngHour = lon / 15.0;
  float t =  N + ((rise ? 6.0 : 18.0) - lngHour) / 24.0;
  float M = (0.9856 * t) - 3.289;
  float L = M + (1.916 * sin(M * toRad)) + (0.020 * sin(2 * M * toRad)) + 282.634;
  L += (L < 0 ? 360 : L > 360 ? -360 : 0); 
  float RA = atan(0.91764 * tan(L * toRad)) * toDeg;
  RA += (RA < 0 ? 360 : RA > 360 ? -360 : 0);
  float Lquadrant = (floor(L / 90.0)) * 90.0;
  float RAquadrant = (floor(RA / 90.0)) * 90.0;
  RA = (RA + Lquadrant - RAquadrant) / 15.0;
  float sinDec = 0.39782 * sin(L * toRad);
  float cosDec = cos(asin(sinDec));
  float cosH = (cos(zenith * toRad) - (sinDec * sin(lat * toRad))) / (cosDec * cos(lat * toRad));
  float H = (rise ? (360.0 - acos(cosH) * toDeg) : (acos(cosH) * toDeg)) / 15; 
  float T = H + RA - (0.06571 * t) - 6.622;
  float UT = T - lngHour;
  UT +=  (UT < 0 ? 24 : UT > 24 ? -24 : 0);
  return UT;  // UTC decimal time 
}
